"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
Object.defineProperty(exports, "__esModule", { value: true });
var algorithm_1 = require("@phosphor/algorithm");
var VDOM_MIME_TYPE = 'application/vdom.v1+json';
var HTML_MIME_TYPE = 'text/html';
/**
 * Create a TOC generator for notebooks.
 *
 * @param tracker: A notebook tracker.
 *
 * @returns A TOC generator that can parse notebooks.
 */
function createNotebookGenerator(tracker, sanitizer) {
    return {
        tracker: tracker,
        usesLatex: true,
        generate: function (panel) {
            var headings = [];
            algorithm_1.each(panel.notebook.widgets, function (cell) {
                var model = cell.model;
                // Only parse markdown cells or code cell outputs
                if (model.type === 'code') {
                    // Iterate over the outputs, and parse them if they
                    // are rendered markdown or HTML.
                    for (var i = 0; i < model.outputs.length; i++) {
                        // Filter out the outputs that are not rendered HTML
                        // (that is, markdown, vdom, or text/html)
                        var outputModel = model.outputs.get(i);
                        var dataTypes = Object.keys(outputModel.data);
                        var htmlData = dataTypes.filter(function (t) { return Private.isMarkdown(t) || Private.isDOM(t); });
                        if (!htmlData.length) {
                            continue;
                        }
                        // If the output has rendered HTML, parse it for headers.
                        var outputWidget = cell.outputArea.widgets[i];
                        var onClickFactory = function (el) {
                            return function () {
                                el.scrollIntoView();
                            };
                        };
                        headings = headings.concat(Private.getRenderedHTMLHeadings(outputWidget.node, onClickFactory, sanitizer));
                    }
                }
                else if (model.type === 'markdown') {
                    // If the cell is rendered, generate the ToC items from
                    // the HTML. If it is not rendered, generate them from
                    // the text of the cell.
                    if (cell.rendered) {
                        var onClickFactory = function (el) {
                            return function () {
                                if (!cell.rendered) {
                                    cell.node.scrollIntoView();
                                }
                                else {
                                    el.scrollIntoView();
                                }
                            };
                        };
                        headings = headings.concat(Private.getRenderedHTMLHeadings(cell.node, onClickFactory, sanitizer));
                    }
                    else {
                        var onClickFactory = function (line) {
                            return function () {
                                cell.node.scrollIntoView();
                                if (!cell.rendered) {
                                    cell.editor.setCursorPosition({ line: line, column: 0 });
                                }
                            };
                        };
                        headings = headings.concat(Private.getMarkdownHeadings(model.value.text, onClickFactory));
                    }
                }
            });
            return headings;
        },
    };
}
exports.createNotebookGenerator = createNotebookGenerator;
/**
 * Create a TOC generator for markdown files.
 *
 * @param tracker: A file editor tracker.
 *
 * @returns A TOC generator that can parse markdown files.
 */
function createMarkdownGenerator(tracker) {
    return {
        tracker: tracker,
        usesLatex: true,
        isEnabled: function (editor) {
            // Only enable this if the editor mimetype matches
            // one of a few markdown variants.
            return Private.isMarkdown(editor.model.mimeType);
        },
        generate: function (editor) {
            var model = editor.model;
            var onClickFactory = function (line) {
                return function () {
                    editor.editor.setCursorPosition({ line: line, column: 0 });
                };
            };
            return Private.getMarkdownHeadings(model.value.text, onClickFactory);
        },
    };
}
exports.createMarkdownGenerator = createMarkdownGenerator;
/**
 * Create a TOC generator for LaTeX files.
 *
 * @param tracker: A file editor tracker.
 *
 * @returns A TOC generator that can parse LaTeX files.
 */
function createLatexGenerator(tracker) {
    return {
        tracker: tracker,
        usesLatex: true,
        isEnabled: function (editor) {
            // Only enable this if the editor mimetype matches
            // one of a few LaTeX variants.
            var mime = editor.model.mimeType;
            return mime === 'text/x-latex' || mime === 'text/x-stex';
        },
        generate: function (editor) {
            var headings = [];
            var model = editor.model;
            // Split the text into lines, with the line number for each.
            // We will use the line number to scroll the editor upon
            // TOC item click.
            var lines = model.value.text.split('\n').map(function (value, idx) {
                return { value: value, idx: idx };
            });
            // Iterate over the lines to get the header level and
            // the text for the line.
            lines.forEach(function (line) {
                var match = line.value.match(/^\s*\\(section|subsection|subsubsection){(.+)}/);
                if (match) {
                    var level = Private.latexLevels[match[1]];
                    var text = match[2];
                    var onClick = function () {
                        editor.editor.setCursorPosition({ line: line.idx, column: 0 });
                    };
                    headings.push({ text: text, level: level, onClick: onClick });
                }
            });
            return headings;
        },
    };
}
exports.createLatexGenerator = createLatexGenerator;
/**
 * A private namespace for miscellaneous things.
 */
var Private;
(function (Private) {
    /**
     * Given a string of markdown, get the markdown headings
     * in that string.
     */
    function getMarkdownHeadings(text, onClickFactory) {
        // Split the text into lines.
        var lines = text.split('\n');
        var headings = [];
        // Iterate over the lines to get the header level and
        // the text for the line.
        lines.forEach(function (line, idx) {
            // Make an onClick handler for this line.
            var onClick = onClickFactory(idx);
            // First test for '#'-style headers.
            var match = line.match(/^([#]{1,6}) (.*)/);
            if (match) {
                var level = match[1].length;
                // Take special care to parse markdown links into raw text.
                var text_1 = match[2].replace(/\[(.+)\]\(.+\)/g, '$1');
                headings.push({ text: text_1, level: level, onClick: onClick });
                return;
            }
            // Next test for '==='-style headers.
            match = line.match(/^([=]{2,}|[-]{2,})/);
            if (match && idx > 0) {
                var level = match[1][0] === '=' ? 1 : 2;
                // Take special care to parse markdown links into raw text.
                var text_2 = lines[idx - 1].replace(/\[(.+)\]\(.+\)/g, '$1');
                headings.push({ text: text_2, level: level, onClick: onClick });
                return;
            }
            // Finally test for HTML headers. This will not catch multiline
            // headers, nor will it catch multiple headers on the same line.
            // It should do a decent job of catching many, though.
            match = line.match(/<h([1-6])>(.*)<\/h\1>/i);
            if (match) {
                var level = parseInt(match[1]);
                var text_3 = match[2];
                headings.push({ text: text_3, level: level, onClick: onClick });
            }
        });
        return headings;
    }
    Private.getMarkdownHeadings = getMarkdownHeadings;
    /**
     * Given an HTML element, generate ToC headings
     * by finding all the headers and making IHeading objects for them.
     */
    function getRenderedHTMLHeadings(node, onClickFactory, sanitizer) {
        var headings = [];
        var headingNodes = node.querySelectorAll('h1, h2, h3, h4, h5, h6');
        for (var i = 0; i < headingNodes.length; i++) {
            var heading = headingNodes[i];
            var level = parseInt(heading.tagName[1]);
            var text = heading.textContent;
            var html = sanitizer.sanitize(heading.innerHTML, sanitizerOptions);
            html = html.replace('¶', ''); // Remove the anchor symbol.
            var onClick = onClickFactory(heading);
            headings.push({ level: level, text: text, html: html, onClick: onClick });
        }
        return headings;
    }
    Private.getRenderedHTMLHeadings = getRenderedHTMLHeadings;
    /**
     * Return whether the mime type is some flavor of markdown.
     */
    function isMarkdown(mime) {
        return (mime === 'text/x-ipythongfm' ||
            mime === 'text/x-markdown' ||
            mime === 'text/x-gfm' ||
            mime === 'text/markdown');
    }
    Private.isMarkdown = isMarkdown;
    /**
     * Return whether the mime type is DOM-ish (html or vdom).
     */
    function isDOM(mime) {
        return (mime === VDOM_MIME_TYPE ||
            mime === HTML_MIME_TYPE);
    }
    Private.isDOM = isDOM;
    /**
     * A mapping from LaTeX section headers to HTML header
     * levels. `part` and `chapter` are less common in my experience,
     * so assign them to header level 1.
     */
    Private.latexLevels = {
        part: 1,
        chapter: 1,
        section: 1,
        subsection: 2,
        subsubsection: 3,
        paragraph: 4,
        subparagraph: 5,
    };
    /**
     * Allowed HTML tags for the ToC entries. We use this to
     * sanitize HTML headings, if they are given. We specifically
     * disallow anchor tags, since we are adding our own.
     */
    var sanitizerOptions = {
        allowedTags: [
            'p',
            'blockquote',
            'b',
            'i',
            'strong',
            'em',
            'strike',
            'code',
            'br',
            'div',
            'span',
            'pre',
            'del',
        ],
        allowedAttributes: {
            // Allow "class" attribute for <code> tags.
            code: ['class'],
            // Allow "class" attribute for <span> tags.
            span: ['class'],
            // Allow "class" attribute for <div> tags.
            div: ['class'],
            // Allow "class" attribute for <p> tags.
            p: ['class'],
            // Allow "class" attribute for <pre> tags.
            pre: ['class'],
        },
    };
})(Private || (Private = {}));
